@@ -1,4 +1,4 @@
-/* Copyright (C) 2013, 2014, 2018, 2019  Olga Yakovleva <yakovleva.o.v@gmail.com> */
+/* Copyright (C) 2013, 2014, 2018, 2019, 2022  Olga Yakovleva <olga@rhvoice.prg> */ 
 
 /* This program is free software: you can redistribute it and/or modify */
 /* it under the terms of the GNU Lesser General Public License as published by */
@@ -13,6 +13,7 @@
 /* You should have received a copy of the GNU Lesser General Public License */
 /* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
+#include <iostream>
 #include "core/str.hpp"
 #include "core/path.hpp"
 #include "core/config.hpp"
@@ -26,9 +27,12 @@ namespace RHVoice
     data_path(info_.get_data_path()),
     beta("beta",0.4,-0.8,0.8),
     gain("gain",1.0,0.5,2.0),
+    native_rate("rate", 1.0, 0.5, 2.0),
+    voicing("voicing", 0.5, 0, 1.0),
     quality(quality_none),
     int_key("key",200,50,500),
     emph_shift("emph_shift",0,-12,12),
+    base_pitch_shift("pitch_shift",0,-12,12),
     input(0),
     output(0),
     rate(1.0),
@@ -63,9 +67,12 @@ namespace RHVoice
   {
     config cfg2;
     cfg2.register_setting(beta);
+    cfg2.register_setting(voicing);
     cfg2.register_setting(gain);
+    cfg2.register_setting(native_rate);
     cfg2.register_setting(int_key);
     cfg2.register_setting(emph_shift);
+    cfg2.register_setting(base_pitch_shift);
     cfg2.load(path::join(data_path,"voice.params"));
     if(int_key.is_set())
       pitch_editor.set_key(int_key);
