@@ -1484,8 +1484,47 @@ namespace RHVoice
         return is_silence(seg)?x:seg.eval(full_name,zero);
       }
     };
+
+    struct hts_dist_to_prev_round_vowel_in_word: public feature_function
+    {
+      hts_dist_to_prev_round_vowel_in_word():
+        feature_function("dist_to_prev_round_vowel_in_word")
+      {
+      }
+
+      value eval(const item& seg) const
+      {
+        if(is_silence(seg))
+          return x;
+        const item& syl=seg.as("SylStructure").parent();
+        const item& word=syl.parent();
+        item::const_reverse_iterator syl_pos=syl.get_reverse_iterator();
+        item::const_reverse_iterator round_syl_pos=std::find_if(syl_pos,word.rend(),feature_equals<std::string>("syl_vowel_vrnd","+"));
+        unsigned int result=(round_syl_pos==word.rend())?0:(std::distance(syl_pos,round_syl_pos)+1);
+        return result;
+      }
+    };
+
   }
 
+    struct hts_syl_vowel_ph_flag_feat: public feature_function
+    {
+    private:
+      const std::string full_name;
+
+    public:
+      hts_syl_vowel_ph_flag_feat(const std::string& hts_prefix,const std::string& path,const std::string& short_name):
+        feature_function(hts_prefix+"syl_vowel_ph_flag_"+short_name),
+        full_name("R:SylStructure.parent."+path+"syl_vowel_ph_flag_"+short_name)
+      {
+      }
+
+      value eval(const item& seg) const
+      {
+        return is_silence(seg)?x:seg.eval(full_name,zero);
+      }
+    };
+
   void hts_labeller::load_label_format_description(const std::string& file_path)
   {
     std::ifstream infile;
@@ -1646,6 +1685,7 @@ define_feature(std::shared_ptr<feature_function>(new hts_utt_type));
 define_feature(std::shared_ptr<feature_function>(new hts_prev_syl_coda_length));
 define_feature(std::shared_ptr<feature_function>(new hts_syl_coda_length));
 define_feature(std::shared_ptr<feature_function>(new hts_next_syl_coda_length));
+ define_feature(std::shared_ptr<feature_function>(new hts_dist_to_prev_round_vowel_in_word));
   }
 
   void hts_labeller::define_extra_phonetic_feature(const std::string& name)
@@ -1664,5 +1704,17 @@ define_feature(std::shared_ptr<feature_function>(new hts_next_syl_coda_length));
     define_feature(std::shared_ptr<feature_function>(new hts_ph_flag_feat("prev_","p.",name)));
     define_feature(std::shared_ptr<feature_function>(new hts_ph_flag_feat("next_next_","n.n.",name)));
     define_feature(std::shared_ptr<feature_function>(new hts_ph_flag_feat("prev_prev_","p.p.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("","",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("next_in_word_","n.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("prev_in_word_","p.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("next_next_in_word_","n.n.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("prev_prev_in_word_","p.p.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("first_in_word_","parent.daughter1.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("last_in_word_","parent.daughtern.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("next_","R:Syllable.n.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("prev_","R:Syllable.p.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("next_next_","R:Syllable.n.n.",name)));
+    define_feature(std::shared_ptr<feature_function>(new hts_syl_vowel_ph_flag_feat("prev_prev_","R:Syllable.p.p.",name)));
 }
 }
+ 
