@@ -52,7 +52,7 @@ api_version_0=(0,0,0)
 api_version_2019_3=(2019,3,0)
 api_version=api_version_0
 try:
-	import AddonAPIVersion
+	import addonAPIVersion
 	api_version=addonAPIVersion.CURRENT
 except ImportError:
 	pass
@@ -736,7 +736,7 @@ class SynthDriver(synthDriverHandler.SynthDriver):
 	@classmethod
 	def check(cls):
 		for addon in addonHandler.getRunningAddons():
-			if addon.name.startswith("RHVoice-voice"):
+			if re.match("RHVoice.*-voice", addon.name):
 				return True
 		return False
 
